@@ -11,6 +11,7 @@ from utils.qr import QRCode
 from cli.wallet import HandleWalletFunctions
 from conf.meile_config import MeileGuiConfig
 from typedef.win import CoinsList
+from cli.warp import WarpHandler
 
 
 from kivy.uix.screenmanager import Screen, SlideTransition
@@ -23,6 +24,9 @@ from kivymd.uix.card import MDCard
 from kivy.utils import get_color_from_hex
 from kivy.metrics import dp
 from kivymd.uix.menu import MDDropdownMenu
+from kivymd.uix.behaviors import HoverBehavior
+from kivymd.theming import ThemableBehavior
+from kivy.core.window import Window
 
 
 
@@ -34,7 +38,7 @@ from os import path,geteuid
 import sys
 import copy 
 import re
-
+from time import sleep
 
 class WalletRestore(Screen):
     screemanager = ObjectProperty()
@@ -166,11 +170,25 @@ class PreLoadWindow(Screen):
         self.NodeTree = NodeTreeData(None)
         
         self.GenerateUUID()
+        self.CreateWarpConfig()
         
         # Schedule the functions to be called every n seconds
-        Clock.schedule_once(partial(self.NodeTree.get_nodes, "12s"), 3)
+        Clock.schedule_once(partial(self.NodeTree.get_nodes, "13s"), 3)
         Clock.schedule_interval(self.update_status_text, 0.6)
         
+    def CreateWarpConfig(self):
+        MeileConfig = MeileGuiConfig()
+        CONFIG = MeileConfig.read_configuration(MeileGuiConfig.CONFFILE)
+        
+        if 'warp' in CONFIG:
+            return 
+        else:
+            CONFIG['warp'] = {}
+            CONFIG['warp']['registered'] = str(0)
+        with open(MeileGuiConfig.CONFFILE,'w') as FILE:
+            CONFIG.write(FILE)
+        FILE.close()
+        
     def GenerateUUID(self):
         MeileConfig = MeileGuiConfig()
         CONFIG = MeileConfig.read_configuration(MeileGuiConfig.CONFFILE)
@@ -205,9 +223,7 @@ class PreLoadWindow(Screen):
         )
         self.dialog.open()
         
-    def quit_meile(self, dt):
-        sys.exit("Not running as root")
-        
+   
     @delayable
     def update_status_text(self, dt):
         go_button = self.manager.get_screen(WindowNames.PRELOAD).ids.go_button
@@ -250,6 +266,7 @@ class MainWindow(Screen):
     old_ip = ""
     ip = ""
     CONNECTED = None
+    warpd = False
     NodeTree = None
     SubResult = None
     MeileConfig = None
@@ -290,6 +307,10 @@ class MainWindow(Screen):
         self.Sort = text_item
         self.menu.dismiss()
         
+    def set_warp_icon(self):
+        MeileConfig = MeileGuiConfig()
+        return MeileConfig.resource_path("../imgs/warp.png")
+     
     def set_protected_icon(self, setbool, moniker):
         MeileConfig = MeileGuiConfig()
         if setbool:
@@ -322,7 +343,77 @@ class MainWindow(Screen):
             None,
             self.manager.get_screen(WindowNames.MAIN_WINDOW).ids.android_tabs.ids.layout.children[-1].text
         )
-
+        
+    @mainthread
+    def display_warp_success(self):
+        
+        self.dialog = MDDialog(
+            text="You are now using DoH (DNS-over-HTTPS) and your DNS traffic is encrypted from prying eyes.",
+            md_bg_color=get_color_from_hex("#0d021b"),
+            buttons=[
+                MDRaisedButton(
+                    text="Okay",
+                    theme_text_color="Custom",
+                    text_color=(1,1,1,1),
+                    on_release=self.remove_loading_widget
+                ),
+            ],
+        )
+        self.dialog.open()
+        
+    @delayable    
+    def start_warp(self):
+        MeileConfig = MeileGuiConfig()
+        WARP = WarpHandler()
+        CONFIG = MeileConfig.read_configuration(MeileGuiConfig.CONFFILE)
+        self.add_loading_popup("Staring WARP service...")
+        yield 2.3
+        if not self.warpd:
+            if WARP.start_warp_daemon():
+                
+                sleep(4)
+                self.warpd = True
+            
+    
+            if int(CONFIG['warp'].get('registered')) == 0:
+                print("Registering WARP...")
+                CONFIG.set('warp','registered', '1')
+                with open(MeileGuiConfig.CONFFILE, 'w') as FILE:
+                    CONFIG.write(FILE)
+                FILE.close()
+                if WARP.register_warp():
+                    sleep(6)
+                    print("Running WARP...")
+                    if WARP.run_warp():
+                        print("SUCCESS")
+                        sleep(3)
+                        self.remove_loading_widget(None)
+                        self.display_warp_success()
+                        
+            else:
+                print("Running WARP...")
+                if WARP.run_warp():
+                    sleep(3)
+                    print("WARP: Success!")
+                    self.remove_loading_widget(None)
+                    self.display_warp_success()
+            
+        else:
+            #self.remove_loading_widget(None)
+            self.dialog = MDDialog(
+                text="You are already using WARP.",
+                md_bg_color=get_color_from_hex("#0d021b"),
+                buttons=[
+                    MDRaisedButton(
+                        text="OKAY",
+                        theme_text_color="Custom",
+                        text_color=(1,1,1,1),
+                        on_release=self.remove_loading_widget
+                    ),
+                ],
+            )
+            self.dialog.open()
+            
     def get_logo(self):
         self.MeileConfig = MeileGuiConfig()
         return self.MeileConfig.resource_path("../imgs/logo.png")
@@ -467,14 +558,14 @@ class MainWindow(Screen):
             },
         )
             
-    @mainthread        
+    @mainthread
     def add_loading_popup(self, title_text):
         self.dialog = None
         self.dialog = MDDialog(title=title_text,md_bg_color=get_color_from_hex("#0d021b"))
         self.dialog.open()
         
     @mainthread
-    def remove_loading_widget(self,dt):
+    def remove_loading_widget(self, dt):
         try:
             self.dialog.dismiss()
             self.dialog = None
@@ -528,7 +619,7 @@ class MainWindow(Screen):
         self.remove_loading_widget(None)
         
         self.add_loading_popup("Reloading Nodes & Subscriptions...")
-        yield 1.3
+        yield 0.5
         try: 
             self.NodeTree.NodeTree = None
             thread = ThreadWithResult(target=self.NodeTree.get_nodes, args=(latency.return_latency(),)) 
@@ -538,10 +629,10 @@ class MainWindow(Screen):
         except Exception as e:
             print(str(e))
             pass
-        self.GetSubscriptions()
+        self.SubResult = None
         self.remove_loading_widget(None)
         self.ids.android_tabs.switch_tab("Subscriptions")
-        self.on_tab_switch(None, None, None, "Subscriptions")
+        #self.on_tab_switch(None, None, None, "Subscriptions")
         
     def GetSubscriptions(self):
         try: 
@@ -557,14 +648,11 @@ class MainWindow(Screen):
     
     @delayable
     def subs_callback(self, dt):
-        #from src.cli.sentinel import NodesDictList
-         
-        
         floc = "../imgs/"
         yield 0.314
         if not self.SubResult:
             self.GetSubscriptions()
-        #self.Subscriptions = get_subscriptions(NodesDictList, self.address)
+        
         for sub in self.SubResult:
             if sub[FinalSubsKeys[5]] == "Czechia":
                 sub[FinalSubsKeys[5]] = "Czech Republic"
@@ -576,6 +664,7 @@ class MainWindow(Screen):
             self.add_sub_rv_data(sub, flagloc)
         self.remove_loading_widget(None)
 
+
     @mainthread
     def on_tab_switch(self, instance_tabs, instance_tab, instance_tabs_label, tab_text):
         #from src.cli.sentinel import ConNodes, NodesDictList
@@ -587,13 +676,12 @@ class MainWindow(Screen):
         # Subscriptions
         #print(self.NodeTree.NodeTree.show())
         if tab_text == OurWorld.CONTINENTS[6]:
-            self.add_loading_popup("Loading...")
             self.get_config(None)
+            self.add_loading_popup("Loading...")
             if self.address:
-            
+                
                 Clock.schedule_once(self.subs_callback, 1)
                 #Subscriptions = get_subscriptions(NodesDictList, address)
-                
                 return 
             else:
                 self.remove_loading_widget(None)
@@ -855,8 +943,16 @@ class NodeScreen(Screen):
         Meile.app.root.current = WindowNames.MAIN_WINDOW
 
         
-class RecycleViewCountryRow(MDCard):
-    text = StringProperty()
+class RecycleViewCountryRow(MDCard,ThemableBehavior, HoverBehavior):
+    text = StringProperty()    
+    
+    def on_enter(self, *args):
+        self.md_bg_color = get_color_from_hex("#200c3a")
+        Window.set_system_cursor('hand')
+        
+    def on_leave(self, *args):
+        self.md_bg_color = get_color_from_hex("#0d021b")
+        Window.set_system_cursor('arrow')
     
     def show_country_nodes(self, country):
         print(country)
