@@ -12,6 +12,9 @@ from kivy.metrics import dp
 from kivyoav.delayed import delayable
 from kivy.uix.screenmanager import Screen, SlideTransition
 from kivy.utils import get_color_from_hex
+from kivymd.uix.behaviors import HoverBehavior
+from kivymd.theming import ThemableBehavior
+from kivy.core.window import Window
 
 from functools import partial
 from urllib3.exceptions import InsecureRequestWarning
@@ -140,11 +143,30 @@ class SelectableLabel(RecycleDataViewBehavior, Label):
 class NodeRV(RecycleView):    
     pass
 
+class OnHoverMDRaisedButton(MDRaisedButton, ThemableBehavior, HoverBehavior):
+    def on_enter(self, *args):
+        self.md_bg_color = get_color_from_hex("#fad783")
+        Window.set_system_cursor('arrow')
+        
+    def on_leave(self, *args):
+        '''The method will be called when the mouse cursor goes beyond
+        the borders of the current widget.'''
 
-class RecycleViewRow(MDCard):
+        self.md_bg_color = get_color_from_hex("#fcb711")
+        Window.set_system_cursor('arrow')
+
+class RecycleViewRow(MDCard,ThemableBehavior, HoverBehavior):
     text = StringProperty()    
     dialog = None
     
+    def on_enter(self, *args):
+        self.md_bg_color = get_color_from_hex("#200c3a")
+        Window.set_system_cursor('hand')
+        
+    def on_leave(self, *args):
+        self.md_bg_color = get_color_from_hex("#0d021b")
+        Window.set_system_cursor('arrow')
+        
     def get_city_of_node(self, naddress):   
         APIURL   = "https://api.sentinel.mathnodes.com"
 
@@ -307,10 +329,7 @@ Node Version: %s
 class RecycleViewSubRow(MDCard):
     text = StringProperty()
     dialog = None
-    
-    def set_background_color(self, color):
-        self.md_bg_color = color
-    
+        
     def get_data_used(self, allocated, consumed):
         try:         
             allocated = float(allocated.replace('GB',''))
@@ -343,7 +362,11 @@ class RecycleViewSubRow(MDCard):
             print(str(e))
             return
     @delayable
-    def connect_to_node(self, ID, naddress, moniker):
+    def connect_to_node(self, ID, naddress, moniker, switchValue):
+        if switchValue == False:
+            Meile.app.root.get_screen(WindowNames.MAIN_WINDOW).disconnect_from_node()
+            return 
+        
         self.add_loading_popup("Connecting...")
         
         yield 0.6
